import json
import pathlib

## reads global variables and returns values ##

def current_season(type="last_full_week"):
    """ 
    Returns the current season
    """
    global_variables = None
    with open('{0}/global_variables.json'.format(pathlib.Path(__file__).parent.resolve()), 'r') as fp:
        ## load config ##
        global_variables = json.load(fp)
    
    ## return ##
    if type in global_variables['season_states']:
        return global_variables['season_states'][type]['season']
    else:
        raise ValueError('Invalid season type: {0}'.format(type))



import pandas as pd
import numpy
import json

import nfelodcm.nfelodcm.Utilities as utils
import nfelodcm.nfelodcm.Engine.Assignments as assignments

class LocalIO():
    """
    Reads and writes local CSVs provided a map that contains fields and assignments
    """
    
    def __init__(self, config, data_location):
        self.config = config
        self.columns, self.dtypes = utils.extract_map(config['map'])
        self.data_location = data_location
        self.df = None
        self.add_assignment_cols()
    
    def add_assignment_cols(self):
        """
        Add columns to the dataframe based on the assignments in the config
        """
        for assignment in self.config['assignments']:
            cols_added = assignments.assignment_columns_added(assignment)
            for col in cols_added:
                if col[0] not in self.columns:
                    self.columns.append(col[0])
                    self.dtypes[col[0]] = col[1]
      
    def read(self):
        """
        Read the data from local CSV
        """
        self.df = pd.read_csv(
            self.data_location,
            usecols=self.columns,
            dtype=self.dtypes,
            engine=self.config['engine']
        )
        
    def write(self):
        """
        Write the data to local CSV
        """
        if self.df is None:
            raise Exception('Dataframe is None. Cannot write. Please load a DF into the LocalIO object first.')
        self.df.to_csv(
            self.data_location
        )



from .assignments import game_id_repl

## a dictionary that holds assignments and the columns they add ##
assignments = {
    'game_id_repl' : {
        'func' : game_id_repl,
        'new_columns' : []
    }
}

def assign(df, assignment):
    """
    An extension on the pandas assign() function which allows the dcm to add non-standard fields
    to the CSVs it downloads.
    """
    ## get the function ##
    func = assignments[assignment]['func']
    ## return the assignment ##
    return func(df)


def assignment_columns_added(assignment):
    """
    When an assignment is used and adds columns to a dataframe not in the map,
    assignment_columns_added() will returns a list of the columns added, so they can
    be passed when reading from cache
    """
    return assignments[assignment]['new_columns']
